@@ -259,7 +259,9 @@ def box(extents=None, transform=None):
     '''
     vertices = [0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1,
                 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1]
-    vertices = np.array(vertices, dtype=np.float64).reshape((-1, 3))
+    vertices = np.array(vertices,
+                        order='C',
+                        dtype=np.float64).reshape((-1, 3))
     vertices -= 0.5
 
     if extents is not None:
@@ -270,11 +272,14 @@ def box(extents=None, transform=None):
 
     faces = [1, 3, 0, 4, 1, 0, 0, 3, 2, 2, 4, 0, 1, 7, 3, 5, 1, 4,
              5, 7, 1, 3, 7, 2, 6, 4, 2, 2, 7, 6, 6, 5, 4, 7, 5, 6]
-    faces = np.array(faces, dtype=np.int64).reshape((-1, 3))
+    faces = np.array(faces,
+                     order='C', dtype=np.int64).reshape((-1, 3))
 
     face_normals = [-1, 0, 0, 0, -1, 0, -1, 0, 0, 0, 0, -1, 0, 0, 1, 0, -1,
                     0, 0, 0, 1, 0, 1, 0, 0, 0, -1, 0, 1, 0, 1, 0, 0, 1, 0, 0]
-    face_normals = np.array(face_normals, dtype=np.float64).reshape(-1, 3)
+    face_normals = np.asanyarray(face_normals,
+                                 order='C',
+                                 dtype=np.float64).reshape(-1, 3)
 
     box = Trimesh(vertices=vertices,
                   faces=faces,
