@@ -1,3 +1,5 @@
+from .. import util
+
 from .generic import MeshScript
 from ..resources import get_resource
 from ..constants import log
@@ -16,6 +18,13 @@ if platform.system() == 'Windows':
     _search_path = ';'.join(_search_path)
     log.debug('searching for blender in: ', _search_path)
 
+if platform.system() == 'Darwin':
+    _search_path = [i for i in _search_path.split(':') if len(i) > 0]
+    _search_path.append('/Applications/blender.app/Contents/MacOS')
+    _search_path = ':'.join(_search_path)
+    log.debug('searching for blender in: ', _search_path)
+    log.warning('searching for blender in: ', _search_path)
+
 
 _blender_executable = find_executable('blender', path=_search_path)
 _blender_template = get_resource('blender.py.template')
@@ -37,6 +46,8 @@ def boolean(meshes, operation='difference'):
         result = blend.run(_blender_executable +
                            ' --background --python $script')
 
-    # blender returns actively incorrect face normals
-    result['face_normals'] = None
+    for m in util.make_sequence(result):
+        # blender returns actively incorrect face normals
+        m.face_normals = None
+
     return result
