@@ -24,14 +24,20 @@ for _name in ['vhacd', 'testVHACD']:
 
 exists = _vhacd_executable is not None
 
-def convex_decomposition(mesh):
+
+def convex_decomposition(mesh, **kwargs):
     if not exists:
         raise ValueError('No vhacd available!')
 
     argstring = ' --input $mesh_0 --output $mesh_post --log $script'
+
+    # pass through extra arguments from the input dictionary
+    for key, value in kwargs.items():
+        argstring += ' --{} {}'.format(str(key),
+                                       str(value))
+
     with MeshScript(meshes=[mesh],
                     script='',
                     tmpfile_ext='obj') as vhacd:
         result = vhacd.run(_vhacd_executable + argstring)
     return result
-
