@@ -7,11 +7,11 @@ A library designed to work with vector paths.
 import numpy as np
 import networkx as nx
 
+import copy
 import collections
 
 from shapely.geometry import Polygon
 from scipy.spatial import cKDTree as KDTree
-from copy import deepcopy
 
 from ..points import plane_fit
 from ..geometry import plane_transform
@@ -20,7 +20,6 @@ from ..util import decimal_to_digits
 from ..constants import log
 from ..constants import tol_path as tol
 
-
 from .. import util
 from .. import grouping
 from .. import transformations
@@ -28,17 +27,19 @@ from .. import transformations
 from . import simplify
 from . import entities
 from . import polygons
+from . import traversal
 
-from .traversal import vertex_graph, closed_paths, discretize_path
 from .io.export import export_path
 
 
 class Path(object):
     '''
-    A Path object consists of two things:
+    A Path object consists of:
+
     vertices: (n,[2|3]) coordinates, stored in self.vertices
-    entities: geometric primitives (lines, arcs, and circles)
-              that reference indices in self.vertices
+
+    entities: geometric primitives (aka Lines, Arcs, etc.)
+              that reference indexes in self.vertices
     '''
 
     def __init__(self,
@@ -47,12 +48,16 @@ class Path(object):
                  metadata=None,
                  process=True):
         '''
-        entities:
-            Objects which contain things like keypoints, as
-            references to self.vertices
-        vertices:
-            (n, (2|3)) list of vertices
+        Instantiate a path object.
+
+        Parameters
+        -----------
+        entities: list of Entity objects
+        vertices: (n, dimension) float, vertices referenced by entities
+        metadata: dict, any metadata about the path
+        process:  bool, if True run simple cleanup operations
         '''
+
         self.entities = np.array(entities)
         self.vertices = vertices
         self.metadata = dict()
@@ -82,9 +87,20 @@ class Path(object):
         self._vertices = util.tracked_array(values)
 
     def md5(self):
-        result = self.vertices.md5()
-        result += str(len(self.entities))
-        return result
+        '''
+        What is an MD5 hash of the current vertex and entity arrangment.
+
+        Not robust between loads; use self.identifier_md5 for that
+
+        Returns
+        ------------
+        md5: str, MD5 of current paths
+        '''
+        target = [e.points for e in self.entities]
+        target.append(int(self.vertices.md5()[:10], 16))
+        target = np.hstack(target).astype(np.int32)
+        hashed = util.md5_object(target.tostring())
+        return hashed
 
     @util.cache_decorator
     def paths(self):
@@ -95,38 +111,111 @@ class Path(object):
         ---------
         paths: (n,) sequence of (*,) int referencing self.entities
         '''
-        paths = closed_paths(self.entities, self.vertices)
+        paths = traversal.closed_paths(self.entities, self.vertices)
         return paths
 
+    @util.cache_decorator
+    def dangling(self):
+        '''
+        List of entities that aren't included in a closed path
+
+        Returns
+        ----------
+        dangling: (n,) int, index of self.entities
+        '''
+        dangling = np.setdiff1d(np.arange(len(self.entities)),
+                                np.hstack(self.paths))
+        return dangling
+
     @util.cache_decorator
     def kdtree(self):
+        '''
+        A KDTree object holding the vertices of the path.
+
+        Returns
+        ----------
+        kdtree: scipy.spatial.cKDTree object holding self.vertices
+        '''
+
         kdtree = KDTree(self.vertices.view(np.ndarray))
         return kdtree
 
     @property
     def scale(self):
+        '''
+        What is a representitive number that reflects the magnitude
+        of the world holding the paths, for numerical comparisons.
+
+        Returns
+        ----------
+        scale: float, approximate size of the world holding this path
+        '''
         scale = self.extents.max()
         return scale
 
     @util.cache_decorator
     def bounds(self):
-        return np.vstack((np.min(self.vertices, axis=0),
-                          np.max(self.vertices, axis=0)))
+        '''
+        Return the axis aligned bounding box of the current path.
+
+        Returns
+        ----------
+        bounds: (2, dimension) float, (min, max) coordinates
+        '''
+        # get the bounds of each entity
+        # some entities (mostly Arc) have bounds that differ from their
+        # vertices
+        points = np.array([e.bounds(self.vertices) for e in self.entities])
+        points = points.reshape((-1, self.vertices.shape[1]))
+
+        # get the max and min of all bounds
+        bounds = np.array([points.min(axis=0), points.max(axis=0)])
+
+        return bounds
 
     @property
     def extents(self):
+        '''
+        The size of the axis aligned bounding box
+
+        Returns
+        ---------
+        extents: (dimension,) float, edge length of AABB
+        '''
         return np.diff(self.bounds, axis=0)[0]
 
     @property
     def units(self):
+        '''
+        If there are units defined in self.metadata return them.
+
+        Returns
+        -----------
+        units: str, current unit system
+        '''
         if 'units' in self.metadata:
             return self.metadata['units']
         else:
             return None
 
+    @units.setter
+    def units(self, units):
+        self.metadata['units'] = units
+
+    def convert_units(self, desired, guess=False):
+        '''
+        Convert the units of the current drawing in place.
+
+        Parameters
+        -----------
+        desired: str, unit system to convert to
+        guess:   bool, if True will attempt to guess units
+        '''
+        _set_units(self, desired, guess)
+
     def explode(self):
         '''
-        Turn every multi- segment entity into single segment entities.
+        Turn every multi- segment entity into single segment entities, in- place
         '''
         new_entities = collections.deque()
         for entity in self.entities:
@@ -136,7 +225,7 @@ class Path(object):
     def fill_gaps(self, max_distance=np.inf):
         '''
         Find vertexes with degree 1 and try to connect them to other
-        vertices of degree 1.
+        vertices of degree 1, in place.
 
         Parameters
         ----------
@@ -162,36 +251,78 @@ class Path(object):
     @property
     def is_closed(self):
         '''
-        Are all entities connected to other entities
+        Are all entities connected to other entities.
+
+        Returns
+        -----------
+        closed: every entity is connected at its ends
         '''
         closed = all(i == 2 for i in dict(self.vertex_graph.degree()).values())
         return closed
 
     @util.cache_decorator
     def vertex_graph(self):
-        graph, closed = vertex_graph(self.entities)
+        '''
+        Return a networkx.Graph object for the entity connectiviy
+
+        graph: networkx.Graph object, holding vertex indexes
+        '''
+        graph, closed = traversal.vertex_graph(self.entities)
         return graph
 
-    @units.setter
-    def units(self, units):
-        self.metadata['units'] = units
-
-    def convert_units(self, desired, guess=False):
-        _set_units(self, desired, guess)
-
     def apply_transform(self, transform):
+        '''
+        Apply a transformation matrix to the current path in- place
+
+        Parameters
+        -----------
+        transform: (dimension + 1, dimension + 1) float, homogenous
+                   transformation matrix
+        '''
+        dimension = self.vertices.shape[1]
+        transform = np.asanyarray(transform, dtype=np.float64)
+        if transform.shape != (dimension + 1, dimension + 1):
+            raise ValueError('transform is incorrect shape!')
+        elif np.allclose(transform, np.eye(dimension + 1)):
+            return
+
         self.vertices = transformations.transform_points(self.vertices,
                                                          transform)
+        # explictly clear the cache
+        self._cache.clear()
+
+    def apply_scale(self, scale):
+        '''
+        Apply a transformation matrix to the current path in- place
+
+        Parameters
+        -----------
+        transform: (dimension + 1, dimension + 1) float, homogenous
+                   transformation matrix
+        '''
+        dimension = self.vertices.shape[1]
+        matrix = np.eye(dimension + 1)
+        matrix[:dimension, :dimension] *= float(scale)
+        self.apply_transform(matrix)
 
     def rezero(self):
         '''
-        Translate so that every vertex is positive.
+        Translate so that every vertex is positive in the current mesh is positive.
+
+        Returns
+        -----------
+        matrix: (dimension + 1, dimension + 1) float, homogenous transformation
+                 that was applied to the current Path object.
         '''
-        self.vertices -= self.vertices.min(axis=0)
+        dimension = self.vertices.shape[1]
+        matrix = np.eye(dimension + 1)
+        matrix[:dimension, dimension] = -self.vertices.min(axis=0)
+        self.apply_transform(matrix)
+        return matrix
 
     def merge_vertices(self):
         '''
-        Merges vertices which are identical and replaces references
+        Merges vertices which are identical and replaces references in place.
         '''
         digits = decimal_to_digits(tol.merge * self.scale, min_digits=1)
         unique, inverse = grouping.unique_rows(self.vertices, digits=digits)
@@ -207,7 +338,11 @@ class Path(object):
 
     def remove_entities(self, entity_ids):
         '''
-        Remove entities by their index.
+        Remove entities by index.
+
+        Parameters
+        -----------
+        entity_ids: (n,) int, indexes of self.entities to remove
         '''
         if len(entity_ids) == 0:
             return
@@ -245,13 +380,33 @@ class Path(object):
 
     def discretize_path(self, path):
         '''
-        Return a (n, dimension) list of vertices.
-        Samples arcs/curves to be line segments
+        Given a list of entities, return a list of connected points.
+
+        Parameters
+        -----------
+        path: (n,) int, indexes of self.entities
+
+        Returns
+        -----------
+        discrete: (m, dimension)
         '''
-        discrete = discretize_path(self.entities,
-                                   self.vertices,
-                                   path,
-                                   scale=self.scale)
+        discrete = traversal.discretize_path(self.entities,
+                                             self.vertices,
+                                             path,
+                                             scale=self.scale)
+        return discrete
+
+    @util.cache_decorator
+    def discrete(self):
+        '''
+        A sequence of connected vertices in space, corresponding to
+        self.paths.
+
+        Returns
+        ---------
+        discrete: (len(self.paths),) sequence of (m*, dimension) float
+        '''
+        discrete = np.array([self.discretize_path(i) for i in self.paths])
         return discrete
 
     def paths_to_splines(self, path_indexes=None, smooth=.0002):
@@ -291,7 +446,14 @@ class Path(object):
         return export_dict
 
     def copy(self):
-        return deepcopy(self)
+        '''
+        Get a copy of the current mesh
+
+        Returns
+        ---------
+        copied: Path object, copy of self
+        '''
+        return copy.deepcopy(self)
 
     def show(self):
         if self.is_closed:
@@ -300,13 +462,14 @@ class Path(object):
             self.plot_entities(show=True)
 
     def __add__(self, other):
-        new_entities = deepcopy(other.entities)
+        new_entities = copy.deepcopy(other.entities)
         for entity in new_entities:
             entity.points += len(self.vertices)
-        new_entities = np.append(deepcopy(self.entities), new_entities)
-
-        new_vertices = np.vstack((self.vertices, other.vertices))
-        new_meta = deepcopy(self.metadata)
+        new_entities = np.append(copy.deepcopy(self.entities),
+                                 new_entities)
+        new_vertices = np.vstack((self.vertices,
+                                  other.vertices))
+        new_meta = copy.deepcopy(self.metadata)
         new_meta.update(other.metadata)
 
         new_path = self.__class__(entities=new_entities,
@@ -324,11 +487,6 @@ class Path3D(Path):
                 self.generate_closed_paths,
                 self.generate_discrete]
 
-    @util.cache_decorator
-    def discrete(self):
-        discrete = list(map(self.discretize_path, self.paths))
-        return discrete
-
     def to_planar(self, to_2D=None, normal=None, check=True):
         '''
         Check to see if current vectors are all coplanar.
@@ -362,9 +520,9 @@ class Path3D(Path):
             log.error('points have z with deviation %f', np.std(flat[:, 2]))
             raise NameError('Points aren\'t planar!')
 
-        planar = Path2D(entities=deepcopy(self.entities),
+        planar = Path2D(entities=copy.deepcopy(self.entities),
                         vertices=flat[:, 0:2],
-                        metadata=deepcopy(self.metadata))
+                        metadata=copy.deepcopy(self.metadata))
         to_3D = np.linalg.inv(to_2D)
 
         return planar, to_3D
@@ -422,6 +580,7 @@ class Path2D(Path):
                 self.polygons_closed[self.root[0]])
             self.apply_transform(matrix)
 
+            return matrix
         else:
             raise ValueError('Not implemented for multibody geometry')
 
@@ -437,11 +596,11 @@ class Path2D(Path):
         -----------
         path_3D: Path3D version of current path
         '''
-        vertices_new = np.column_stack((deepcopy(self.vertices),
+        vertices_new = np.column_stack((copy.deepcopy(self.vertices),
                                         np.zeros(len(self.vertices))))
-        path_3D = Path3D(entities=deepcopy(self.entities),
+        path_3D = Path3D(entities=copy.deepcopy(self.entities),
                          vertices=vertices_new,
-                         metadata=deepcopy(self.metadata))
+                         metadata=copy.deepcopy(self.metadata))
         return path_3D
 
     @util.cache_decorator
@@ -455,20 +614,14 @@ class Path2D(Path):
         full: list of shapely.geometry.Polygon objects
         '''
         full = [None] * len(self.enclosure_shell)
-
-        for index, (shell, hole) in enumerate(self.enclosure_shell.items()):
-            if not self.polygons_valid[shell]:
-                continue
-            # mask indexes of holes by valid polygons
-            # this will silently discard invalid holes
-            hole_idx = hole[self.polygons_valid[hole]]
-            # closed curves which represent holes
-            hole_poly = self.polygons_closed[hole_idx]
+        for i, (shell, hole) in enumerate(self.enclosure_shell.items()):
+            hole_poly = self.polygons_closed[hole]
             # generate a new polygon with shell and holes
+            shell_poly = self.polygons_closed[shell]
             polygon = Polygon(
-                shell=self.polygons_closed[shell].exterior.coords, holes=[
-                    p.exterior.coords for p in hole_poly])
-            full[index] = polygon
+                shell=shell_poly.exterior.coords,
+                holes=[p.exterior.coords for p in hole_poly])
+            full[i] = polygon
         return full
 
     @util.cache_decorator
@@ -480,16 +633,16 @@ class Path2D(Path):
         return area
 
     @util.cache_decorator
-    def perimeter(self):
+    def length(self):
         '''
-        The total length of every closed curve.
+        The total discretized length of every entity.
 
         Returns
         --------
-        perimeter: float, length of every boundary (inside and out)
+        length: float, summed length of every entity
         '''
-        perimeter = np.sum([i.length for i in self.polygons_closed])
-        return perimeter
+        length = sum(i.length(self.vertices) for i in self.entities)
+        return length
 
     def extrude(self, height, **kwargs):
         '''
@@ -598,34 +751,36 @@ class Path2D(Path):
         if self.root is None or len(self.root) == 0:
             split = []
         elif len(self.root) == 1:
-            split = [deepcopy(self)]
+            split = [copy.deepcopy(self)]
         else:
             split = [None] * len(self.root)
             for i, root in enumerate(self.root):
                 connected = self.connected_paths(root, include_self=True)
+
                 new_root = np.nonzero(connected == root)[0]
                 new_entities = collections.deque()
                 new_paths = collections.deque()
-                new_metadata = {'split_2D': i}
-                new_metadata.update(self.metadata)
+                new_metadata = copy.deepcopy(self.metadata)
+                new_metadata['split_2D'] = i
 
-                for path in self.paths[connected]:
+                for path in self.paths[self.path_valid][connected]:
                     new_paths.append(np.arange(len(path)) + len(new_entities))
                     new_entities.extend(path)
                 new_entities = np.array(new_entities)
+
+                assert len(new_paths) == len(connected)
+
                 # prevents the copying from nuking our cache
                 with self._cache:
-                    split[i] = Path2D(
-                        entities=deepcopy(
-                            self.entities[new_entities]), vertices=deepcopy(
-                            self.vertices))
+                    split[i] = Path2D(entities=copy.deepcopy(self.entities[new_entities]),
+                                      vertices=copy.deepcopy(self.vertices),
+                                      metadata=new_metadata)
                     split[i]._cache.update(
-                        {
-                            'paths': np.array(new_paths),
-                            'polygons_closed': self.polygons_closed[connected],
-                            'polygons_valid': self.polygons_valid[connected],
-                            'discrete': self.discrete[connected],
-                            'root': new_root})
+                        {'paths': np.array(new_paths),
+                         'polygons_closed': self.polygons_closed[connected],
+                         'path_valid': np.ones(len(new_paths), dtype=np.bool),
+                         'discrete': self.discrete[self.path_valid][connected],
+                         'root': new_root})
         [i._cache.id_set() for i in split]
         self._cache.id_set()
         return np.array(split)
@@ -696,26 +851,15 @@ class Path2D(Path):
         return util.md5_array(self.identifier, digits=4)
 
     @property
-    def polygons_valid(self):
+    def path_valid(self):
         '''
         Returns
         ----------
-        polygons_valid: (n,) bool, indexes of self.paths self.polygons_closed
+        path_valid: (n,) bool, indexes of self.paths self.polygons_closed
                          which are valid polygons
         '''
         exists = self.polygons_closed
-        return self._cache.get('polygons_valid')
-
-    @property
-    def discrete(self):
-        '''
-        Returns
-        ---------
-        discrete: sequence of (n,2) float, correspond to self.paths
-        '''
-        if 'discrete' not in self._cache:
-            test = self.polygons_closed
-        return self._cache['discrete']
+        return self._cache.get('path_valid')
 
     @util.cache_decorator
     def polygons_closed(self):
@@ -729,46 +873,11 @@ class Path2D(Path):
         ---------
         polygons_closed: (n,) list of shapely.geometry.Polygon objects
         '''
-        def reverse_path(path):
-            for entity in self.entities[path]:
-                entity.reverse()
-            return path[::-1]
 
-        with self._cache:
-            discretized = [None] * len(self.paths)
-            new_polygon = [None] * len(self.paths)
-            valid = np.zeros(len(self.paths), dtype=np.bool)
-            for i, path in enumerate(self.paths):
-                discrete = discretize_path(self.entities,
-                                           self.vertices,
-                                           path,
-                                           scale=self.scale)
-                candidate = polygons.path_to_polygon(discrete,
-                                                     scale=self.scale)
-                if candidate is None:
-                    continue
-                if type(candidate).__name__ == 'MultiPolygon':
-                    area_ok = np.array([i.area for i in candidate]) > tol.zero
-                    if area_ok.sum() == 1:
-                        candidate = candidate[np.nonzero(area_ok)[0][0]]
-                    else:
-                        continue
-                if not candidate.exterior.is_ccw:
-                    log.debug('Clockwise polygon detected, correcting!')
-                    self.paths[i] = reverse_path(path)
-                    candidate = Polygon(
-                        np.array(candidate.exterior.coords)[::-1])
-                new_polygon[i] = candidate
-                valid[i] = True
-                discretized[i] = discrete
+        polys, valid = polygons.paths_to_polygons(self.discrete)
+        self._cache.set('path_valid', valid)
 
-            new_polygon = np.array(new_polygon)[valid]
-            discretized = np.array(discretized)
-
-        self._cache.set('discrete', discretized)
-        self._cache.set('polygons_valid', valid)
-
-        return new_polygon
+        return polys
 
     @util.cache_decorator
     def root(self):
@@ -797,8 +906,7 @@ class Path2D(Path):
         '''
         Networkx DiGraph of polygon enclosure
         '''
-        root, enclosure = polygons.polygons_enclosure_tree(
-            self.polygons_closed)
+        root, enclosure = polygons.enclosure_tree(self.polygons_closed)
         self._cache.set('root', root)
         return enclosure
 
