@@ -4,10 +4,13 @@ import copy
 
 from .. import util
 from .. import units
+from .. import convex
 from .. import grouping
+from .. import transformations
 
-from ..bounds import corners as bounds_corners
-from ..transformations import rotation_matrix, transform_points
+from .. import bounds as bounds_module
+
+from ..io import gltf
 from .transforms import TransformForest
 
 
@@ -33,9 +36,8 @@ class Scene:
 
         self._cache = util.Cache(id_function=self.md5)
 
-        if geometry is not None:
-            self.add_geometry(geometry)
-            self.set_camera()
+        self.add_geometry(geometry)
+        self.set_camera()
 
         self.metadata = {}
         self.metadata.update(metadata)
@@ -52,11 +54,14 @@ class Scene:
         Parameters
         ----------
         geometry: Trimesh, Path3D, or list of same
-        node_name:
+        node_name: name in the scene graph
+
         Returns
         ----------
         node_name: str, name of node in self.graph
         '''
+        if geometry is None:
+            return
 
         # if passed a sequence call add_geometry on all elements
         if util.is_sequence(geometry):
@@ -106,6 +111,19 @@ class Scene:
 
         return hashed
 
+    @property
+    def is_empty(self):
+        '''
+        Does the scene have anything in it.
+
+        Returns
+        ----------
+        is_empty: bool, True if nothing is in the scene
+        '''
+
+        is_empty = len(self.geometry) == 0
+        return is_empty
+
     @util.cache_decorator
     def bounds(self):
         '''
@@ -128,10 +146,10 @@ class Scene:
             # geometry objects have bounds properties, which are (2,3) or (2,2)
             current_bounds = self.geometry[geometry_name].bounds.copy()
             # find the 8 corner vertices of the axis aligned bounding box
-            current_corners = bounds_corners(current_bounds)
+            current_corners = bounds_module.corners(current_bounds)
             # transform those corners into where the geometry is located
-            corners.extend(transform_points(current_corners,
-                                            transform))
+            corners.extend(transformations.transform_points(current_corners,
+                                                            transform))
         corners = np.array(corners)
         bounds = np.array([corners.min(axis=0),
                            corners.max(axis=0)])
@@ -157,7 +175,8 @@ class Scene:
         -----------
         scale: float, the mean of the bounding box edge lengths
         '''
-        return self.extents.mean()
+        scale = (self.extents ** 2).sum() ** .5
+        return scale
 
     @util.cache_decorator
     def centroid(self):
@@ -191,10 +210,9 @@ class Scene:
             if not hasattr(geometry, 'triangles'):
                 continue
 
-            triangles.append(
-                transform_points(
-                    geometry.triangles.copy().reshape(
-                        (-1, 3)), transform))
+            triangles.append(transformations.transform_points(
+                geometry.triangles.copy().reshape((-1, 3)),
+                transform))
             triangles_node.append(np.tile(node_name, len(geometry.triangles)))
 
         self._cache['triangles_node'] = np.hstack(triangles_node)
@@ -223,8 +241,12 @@ class Scene:
 
         Returns
         -----------
-        duplicates: (m) sequence of keys to self.nodes that represent identical geometry
+        duplicates: (m) sequence of keys to self.nodes that represent
+                     identical geometry
         '''
+        # if there is no geometry we can have no duplicate nodes
+        if len(self.geometry) == 0:
+            return []
 
         # geometry name : md5 of mesh
         mesh_hash = {k: int(m.identifier_md5, 16)
@@ -257,28 +279,68 @@ class Scene:
         center:    (3,) float, point camera should center on
 
         '''
+        if len(self.geometry) == 0:
+            return
+
         if center is None:
             center = self.centroid
         if distance is None:
-            distance = self.extents.max()
+            # for a 60.0 degree horizontal FOV
+            distance = ((self.extents.max() / 2) /
+                        np.tan(np.radians(60.0) / 2.0))
+
         if angles is None:
             angles = np.zeros(3)
 
         translation = np.eye(4)
         translation[0:3, 3] = center
-        translation[2][3] += distance * 1.5
+        # offset by a distance set by the model size
+        # the FOV is set for the Y axis, we multiply by a lightly
+        # padded aspect ratio to make sure the model is in view initially
+        translation[2][3] += distance * 1.35
 
-        transform = np.dot(rotation_matrix(angles[0], [1, 0, 0], point=center),
-                           rotation_matrix(angles[1], [0, 1, 0], point=center))
+        transform = np.dot(transformations.rotation_matrix(angles[0],
+                                                           [1, 0, 0],
+                                                           point=center),
+                           transformations.rotation_matrix(angles[1],
+                                                           [0, 1, 0],
+                                                           point=center))
         transform = np.dot(transform, translation)
 
         self.graph.update(frame_from='camera',
                           frame_to=self.graph.base_frame,
                           matrix=transform)
 
+    def rezero(self):
+        '''
+        Move the current scene so that the AABB of the whole scene is centered
+        at the origin.
+
+        Does this by changing the base frame to a new, offset base frame.
+        '''
+        if self.is_empty or np.allclose(self.centroid, 0.0):
+            # early exit since what we want already exists
+            return
+
+        # the transformation to move the overall scene to the AABB centroid
+        matrix = np.eye(4)
+        matrix[:3, 3] = -self.centroid
+
+        # we are going to change the base frame
+        new_base = str(self.graph.base_frame) + '_I'
+        self.graph.update(frame_from=new_base,
+                          frame_to=self.graph.base_frame,
+                          matrix=matrix)
+        self.graph.base_frame = new_base
+
     def dump(self):
         '''
         Append all meshes in scene to a list of meshes.
+
+        Returns
+        ----------
+        dumped: (n,) list, of Trimesh objects transformed to their
+                           location the scene.graph
         '''
         result = collections.deque()
 
@@ -290,6 +352,53 @@ class Scene:
             result.append(current)
         return np.array(result)
 
+    @util.cache_decorator
+    def convex_hull(self):
+        '''
+        The convex hull of the whole scene
+
+        Returns
+        ---------
+        hull: Trimesh object, convex hull of all meshes in scene
+        '''
+        points = util.vstack_empty([m.vertices for m in self.dump()])
+        hull = convex.convex_hull(points)
+        return hull
+
+    @util.cache_decorator
+    def bounding_box(self):
+        '''
+        An axis aligned bounding box for the current scene.
+
+        Returns
+        ----------
+        aabb: trimesh.primitives.Box object with transform and extents defined
+              to represent the axis aligned bounding box of the scene
+        '''
+        from .. import primitives
+        center = self.bounds.mean(axis=0)
+        aabb = primitives.Box(transform=transformations.translation_matrix(center),
+                              extents=self.extents,
+                              mutable=False)
+        return aabb
+
+    @util.cache_decorator
+    def bounding_box_oriented(self):
+        '''
+        An oriented bounding box for the current mesh.
+
+        Returns
+        ---------
+        obb: trimesh.primitives.Box object with transform and extents defined
+             to represent the minimum volume oriented bounding box of the mesh
+        '''
+        from .. import primitives
+        to_origin, extents = bounds_module.oriented_bounds(self)
+        obb = primitives.Box(transform=np.linalg.inv(to_origin),
+                             extents=extents,
+                             mutable=False)
+        return obb
+
     def export(self, file_type=None):
         '''
         Export a snapshot of the current scene.
@@ -306,6 +415,12 @@ class Scene:
                 transforms: edge list of transforms, eg:
                              ((u, v, {'matrix' : np.eye(4)}))
         '''
+
+        if file_type == 'gltf':
+            return gltf.export_gltf(self)
+        elif file_type == 'glb':
+            return gltf.export_glb(self)
+
         export = {'graph': self.graph.to_edgelist(),
                   'geometry': {},
                   'scene_cache': {'bounds': self.bounds.tolist(),
@@ -342,46 +457,82 @@ class Scene:
                 export['geometry'][geometry_name] = geometry
         return export
 
-    def save_image(self, file_obj, resolution=(1024, 768), **kwargs):
-        from .viewer import SceneViewer
-        SceneViewer(self,
-                    save_image=file_obj,
-                    resolution=resolution,
-                    **kwargs)
+    def save_image(self, resolution=(1024, 768), **kwargs):
+        '''
+        Get a PNG image of a scene.
 
-    def convert_units(self, desired):
+        Parameters
+        -----------
+        resultion: (2,) int, resolution to render image
+        **kwargs:  passed to SceneViewer constructor
+
+        Returns
+        -----------
+        png: bytes, render of scene in PNG form
+        '''
+        from .viewer import render_scene
+        png = render_scene(scene=self,
+                           resolution=resolution,
+                           **kwargs)
+        return png
+
+    def convert_units(self, desired, guess=False):
         '''
         If geometry has units defined, convert them to new units.
 
+        Returns a new scene with geometries and transforms scaled.
+
         Parameters
         ----------
         units: str, target unit system. EG 'inches', 'mm', etc
         '''
-        existing = np.array([i.units for i in self.geometry.values()])
+        # if there is no geometry do nothing
+        if len(self.geometry) == 0:
+            return self
 
-        if (existing[1] != existing[1:]).any():
+        existing = [i.units for i in self.geometry.values()]
+        if any(existing[0] != e for e in existing):
             # if all of our geometry doesn't have the same units already
             # this function will only do some hot nonsense
             raise ValueError('Models in scene have inconsistent units!')
 
-        scale = units.unit_conversion(current=existing[1],
+        current = existing[0]
+        if current is None:
+            if guess:
+                current = units.unit_guess(self.scale)
+            else:
+                raise ValueError('units not defined and not allowed to guess!')
+
+        # exit early if our current units are the same as desired units
+        if current == desired:
+            return self
+
+        scale = units.unit_conversion(current=current,
                                       desired=desired)
+        result = self.scaled(scale=scale)
 
-        self.graph.scale_transforms(scale)
-
-        for geometry in self.geometry.values():
-            geometry.vertices *= scale
+        for geometry in result.geometry.values():
             geometry.units = desired
 
+        return result
+
     def explode(self, vector=None, origin=None):
         '''
         Explode a scene around a point and vector.
+
+        Parameters
+        -----------
+        vector: (3,) float, or float, explode in a direction or spherically
+        origin: (3,) float, point to explode around
         '''
         if origin is None:
             origin = self.centroid
         if vector is None:
             vector = self.scale / 25.0
 
+        vector = np.asanyarray(vector, dtype=np.float64)
+        origin = np.asanyarray(origin, dtype=np.float64)
+
         centroids = collections.deque()
         for node_name in self.graph.nodes_geometry:
             transform, geometry_name = self.graph[node_name]
@@ -391,19 +542,78 @@ class Scene:
             centroid = np.dot(transform,
                               np.append(centroid, 1))[:3]
 
-            if (isinstance(vector, float) or
-                    isinstance(vector, int)):
+            if vector.shape == ():
+                # case where our vector is a single number
                 offset = (centroid - origin) * vector
             elif np.shape(vector) == (3,):
                 projected = np.dot(vector, (centroid - origin))
                 offset = vector * projected
             else:
-                raise ValueError('vector wrong shape')
+                raise ValueError('explode vector wrong shape!')
 
             transform[0:3, 3] += offset
             self.graph[node_name] = transform
 
+    def scaled(self, scale):
+        '''
+        Return a copy of the current scene, with meshes and scene graph
+        transforms scaled to the requested factor.
+
+        Parameters
+        -----------
+        scale: float, factor to scale meshes and transforms by
+        '''
+        scale = float(scale)
+        scale_matrix = np.eye(4) * scale
+
+        transforms = np.array([self.graph[i][0]
+                               for i in self.graph.nodes_geometry])
+        geometries = np.array([self.graph[i][1]
+                               for i in self.graph.nodes_geometry])
+
+        result = self.copy()
+        result.graph.clear()
+
+        for group in grouping.group(geometries):
+            geometry = geometries[group[0]]
+
+            original = transforms[group[0]]
+            new_geom = np.dot(scale_matrix, original)
+            inv_geom = np.linalg.inv(new_geom)
+
+            result.geometry[geometry].apply_transform(new_geom)
+
+            for node, t in zip(self.graph.nodes_geometry[group],
+                               transforms[group]):
+                transform = util.multi_dot([scale_matrix,
+                                            t,
+                                            np.linalg.inv(new_geom)])
+                transform[:3, 3] *= scale
+
+                result.graph.update(frame_to=node,
+                                    matrix=transform,
+                                    geometry=geometry)
+        return result
+
+    def copy(self):
+        '''
+        Return a deep copy of the current scene
+
+        Returns
+        ----------
+        copied: trimesh.Scene, copy of the current scene
+        '''
+        copied = copy.deepcopy(self)
+        return copied
+
     def show(self, **kwargs):
+        '''
+        Open a pyglet window to preview the current scene
+
+        Parameters
+        -----------
+        smooth: bool, turn on or off automatic smooth shading
+        '''
         # this imports pyglet, and will raise an ImportError
         # if pyglet is not available
         from .viewer import SceneViewer
@@ -425,6 +635,9 @@ def split_scene(geometry):
     if util.is_instance_named(geometry, 'Scene'):
         return geometry
 
+    if util.is_sequence(geometry):
+        return Scene(geometry)
+
     split = collections.deque()
     for g in util.make_sequence(geometry):
         split.extend(g.split())
