@@ -1,6 +1,7 @@
-import numpy as np
 import time
+import collections
 
+import numpy as np
 import networkx as nx
 
 from .. import util
@@ -68,7 +69,7 @@ class TransformForest:
 
         Currently only hashing update time.
         '''
-        result = str(int(self._updated * 1000))
+        result = str(int(self._updated * 1000)) + str(self.base_frame)
         return result
 
     def to_flattened(self, base_frame=None):
@@ -88,6 +89,37 @@ class TransformForest:
                           'geometry': geometry}
         return flat
 
+    def to_gltf(self, mesh_index):
+        '''
+        Export a list of transforms as the 'nodes' section of a GLTF dict.
+
+        Will flatten tree.
+
+        Returns
+        --------
+        gltf: dict, with keys:
+                  'nodes': list of dicts
+        '''
+        gltf = collections.deque()
+
+        for node in self.nodes:
+            if node == self.base_frame:
+                continue
+            transform, geometry = self.get(frame_to=node,
+                                           frame_from=self.base_frame)
+            if geometry is None:
+                continue
+
+            gltf.append({'matrix': transform.T.reshape(-1).tolist(),
+                         'mesh': mesh_index[geometry],
+                         'name': node})
+
+        # we have flattened tree, so all nodes will be child of world
+        gltf.appendleft({'name': self.base_frame,
+                         'children': list(range(1, 1 + len(gltf)))})
+        result = {'nodes': list(gltf)}
+        return result
+
     def to_edgelist(self):
         '''
         Export the current transforms as a list of edge tuples, with
@@ -110,11 +142,12 @@ class TransformForest:
 
     @util.cache_decorator
     def nodes(self):
-        return self.transforms.nodes()
+        return np.array(self.transforms.nodes())
 
     @util.cache_decorator
     def nodes_geometry(self):
-        return [i for i in self.nodes if 'geometry' in self.transforms.node[i]]
+        nodes = [i for i in self.nodes if 'geometry' in self.transforms.node[i]]
+        return np.array(nodes)
 
     def get(self,
             frame_to,
@@ -177,21 +210,8 @@ class TransformForest:
         from ..graph import graph_to_svg
         return graph_to_svg(self.transforms)
 
-    def scale_transforms(self, scale):
-        '''
-        Scale all transforms.
-
-        Parameters
-        -----------
-        scale: float, factor to scale all transformations by
-        '''
-
-        scale = float(scale)
-
-        for a, b in self.transforms.edges():
-            matrix = self.transforms.edge[a][b]['matrix']
-            matrix[:3, 3] *= scale
-            self.transforms.edge[a][b]['matrix'] = matrix
+    def __contains__(self, key):
+        return key in self.transforms.node
 
     def __getitem__(self, key):
         return self.get(key)
@@ -217,14 +237,14 @@ class TransformForest:
         Parameters
         ---------
         frame_from: a frame key, usually a string
-                    example: 'world'
+                    eg, 'world'
         frame_to:   a frame key, usually a string
-                    example: 'mesh_0'
+                    eg, 'mesh_0'
 
         Returns
         ----------
         path: (n) list of frame keys
-              example: ['mesh_finger', 'mesh_hand', 'world']
+              eg, ['mesh_finger', 'mesh_hand', 'world']
         '''
         key = (frame_from, frame_to)
         if not (key in self._paths):
@@ -235,6 +255,10 @@ class TransformForest:
 
 
 class EnforcedForest(nx.DiGraph):
+    '''
+    A subclass of networkx.DiGraph that will raise an error if an
+    edge is added which would make the DiGraph not a forest or tree.
+    '''
 
     def __init__(self, *args, **kwargs):
         self.flags = {'strict': False,
