@@ -13,12 +13,12 @@ import logging
 import hashlib
 import base64
 import time
+import copy
 import json
 import zlib
 
 from sys import version_info
 from functools import wraps
-from copy import deepcopy
 
 _PY3 = version_info.major >= 3
 if _PY3:
@@ -616,10 +616,10 @@ def tracked_array(array, dtype=None):
     '''
     Properly subclass a numpy ndarray to track changes.
     '''
-    result = np.ascontiguousarray(array).view(TrackedArray)
-    if dtype is None:
-        return result
-    return result.astype(dtype)
+    tracked = np.ascontiguousarray(array,
+                                   dtype=dtype).view(TrackedArray)
+    assert tracked.flags['C_CONTIGUOUS']
+    return tracked
 
 
 class TrackedArray(np.ndarray):
@@ -638,11 +638,9 @@ class TrackedArray(np.ndarray):
         This flag will be set on every change, as well as during copies
         and certain types of slicing.
         '''
-        self._modified_md5 = True
-        self._modified_crc = True
+        self._modified = True
         if isinstance(obj, type(self)):
-            obj._modified_md5 = True
-            obj._modified_crc = True
+            obj._modified = True
 
     def md5(self):
         '''
@@ -656,18 +654,35 @@ class TrackedArray(np.ndarray):
         negatives which would return an incorrect hash.
         '''
 
-        if self._modified_md5 or not hasattr(self, '_hashed_md5'):
-            self._hashed_md5 = md5_object(self)
-        self._modified_md5 = False
+        if self._modified or not hasattr(self, '_hashed_md5'):
+            if self.flags['C_CONTIGUOUS']:
+                self._hashed_md5 = md5_object(self)
+            else:
+                # the case where we have sliced our nice
+                # contiguous array into a non- contiguous block
+                # for example (note slice *after* track operation):
+                # t = util.tracked_array(np.random.random(10))[::-1]
+                contiguous = np.ascontiguousarray(self)
+                self._hashed_md5 = md5_object(contiguous)
+        self._modified = False
         return self._hashed_md5
 
     def crc(self):
         '''
         Return a zlib adler32 checksum of the current data.
         '''
-        if self._modified_crc or not hasattr(self, '_hashed_crc'):
-            self._hashed_crc = zlib.adler32(self) & 0xffffffff
-        self._modified_crc = False
+        if self._modified or not hasattr(self, '_hashed_crc'):
+            if self.flags['C_CONTIGUOUS']:
+                self._hashed_crc = zlib.adler32(self) & 0xffffffff
+            else:
+                # the case where we have sliced our nice
+                # contiguous array into a non- contiguous block
+                # for example (note slice *after* track operation):
+                # t = util.tracked_array(np.random.random(10))[::-1]
+                contiguous = np.ascontiguousarray(self)
+                self._hashed_crc = zlib.adler32(contiguous) & 0xffffffff
+
+        self._modified = False
         return self._hashed_crc
 
     def __hash__(self):
@@ -676,14 +691,56 @@ class TrackedArray(np.ndarray):
         '''
         return int(self.md5(), 16)
 
+    def __iadd__(self, other):
+        self._modified = True
+        return super(self.__class__, self).__iadd__(other)
+
+    def __isub__(self, other):
+        self._modified = True
+        return super(self.__class__, self).__isub__(other)
+
+    def __imul__(self, other):
+        self._modified = True
+        return super(self.__class__, self).__imul__(other)
+
+    def __ipow__(self, other):
+        self._modified = True
+        return super(self.__class__, self).__ipow__(other)
+
+    def __imod__(self, other):
+        self._modified = True
+        return super(self.__class__, self).__imod__(other)
+
+    def __ifloordiv__(self, other):
+        self._modified = True
+        return super(self.__class__, self).__ifloordiv__(other)
+
+    def __ilshift__(self, other):
+        self._modified = True
+        return super(self.__class__, self).__ilshift__(other)
+
+    def __irshift__(self, other):
+        self._modified = True
+        return super(self.__class__, self).__irshift__(other)
+
+    def __iand__(self, other):
+        self._modified = True
+        return super(self.__class__, self).__iand__(other)
+
+    def __ixor__(self, other):
+        self._modified = True
+        return super(self.__class__, self).__ixor__(other)
+
+    def __ior__(self, other):
+        self._modified = True
+        return super(self.__class__, self).__ior__(other)
+
     def __setitem__(self, i, y):
-        self._modified_md5 = True
-        self._modified_crc = True
+        self._modified = True
         super(self.__class__, self).__setitem__(i, y)
 
     def __setslice__(self, i, j, y):
-        self._modified_md5 = True
-        self._modified_crc = True
+        self._modified = True
         super(self.__class__, self).__setslice__(i, j, y)
 
 
@@ -762,7 +819,6 @@ class Cache:
         '''
         Update the cache with a set of key, value pairs without checking id_function.
         '''
-        # self.verify()
         self.cache.update(items)
         self.id_set()
 
@@ -836,10 +892,13 @@ class DataStore:
         try:
             return self.data[key]
         except KeyError:
-            return None
+            return np.array([])
 
     def __setitem__(self, key, data):
-        self.data[key] = tracked_array(data)
+        if hasattr(data, 'md5'):
+            self.data[key] = data
+        else:
+            self.data[key] = tracked_array(data)
 
     def __contains__(self, key):
         return key in self.data
@@ -946,7 +1005,8 @@ def append_faces(vertices_seq, faces_seq):
 def array_to_string(array,
                     col_delim=' ',
                     row_delim='\n',
-                    digits=8):
+                    digits=8,
+                    value_format='{}'):
     '''
     Convert a 1 or 2D array into a string with a specified number of digits
     and delimiter.
@@ -968,18 +1028,23 @@ def array_to_string(array,
     digits = int(digits)
     row_delim = str(row_delim)
     col_delim = str(col_delim)
+    value_format = str(value_format)
 
     # abort for non- flat arrays
     if len(array.shape) > 2:
-        raise ValueError('conversion only works on 1D/2D arrays, not %s!',
+        raise ValueError('conversion only works on 1D/2D arrays not %s!',
                          str(array.shape))
 
-    # integer types don't need a specified precision
+    # allow a value to be repeated in a value format
+    repeats = value_format.count('{}')
+
     if array.dtype.kind == 'i':
-        format_str = '{}' + col_delim
-    # for floats use the number of digits we were passed
+        # integer types don't need a specified precision
+        format_str = value_format + col_delim
     elif array.dtype.kind == 'f':
-        format_str = '{:.' + str(digits) + 'f}' + col_delim
+        # add the digits formatting to floats
+        format_str = value_format.replace('{}',
+                                          '{:.' + str(digits) + 'f}') + col_delim
     else:
         raise ValueError('dtype %s not convertable!',
                          array.dtype.name)
@@ -989,14 +1054,20 @@ def array_to_string(array,
     # if we have a 2D array add a row delimiter
     if len(array.shape) == 2:
         format_str *= array.shape[1]
-        format_str += row_delim
-        end_junk += len(row_delim)
+        # cut off the last column delimeter and add a row delimiter
+        format_str = format_str[:-len(col_delim)] + row_delim
+        end_junk = len(row_delim)
 
     # expand format string to whole array
     format_str *= len(array)
 
+    # if an array is repeated in the value format
+    # do the shaping here so we don't need to specify indexes
+    shaped = np.tile(array.reshape((-1, 1)),
+                     (1, repeats)).reshape(-1)
+
     # run the format operation and remove the extra delimiters
-    formatted = format_str.format(*array.reshape(-1))[:-end_junk]
+    formatted = format_str.format(*shaped)[:-end_junk]
 
     return formatted
 
@@ -1269,7 +1340,7 @@ def submesh(mesh,
                            faces=f,
                            face_normals=n,
                            visual=c,
-                           metadata=mesh.metadata,
+                           metadata=copy.deepcopy(mesh.metadata),
                            process=False) for v, f, n, c in zip(vertices,
                                                                 faces,
                                                                 normals,
@@ -1421,7 +1492,7 @@ def bounds_tree(bounds):
     ---------
     tree: Rtree object
     '''
-    bounds = np.asanyarray(deepcopy(bounds), dtype=np.float64)
+    bounds = np.asanyarray(copy.deepcopy(bounds), dtype=np.float64)
     if len(bounds.shape) != 2:
         raise ValueError('Bounds must be (n,dimension*2)!')
 
@@ -1562,7 +1633,7 @@ def sigfig_round(values, sigfig=1):
     rounded: values, but rounded to the specified number of signifigant figures
 
 
-    Example
+    Examples
     ----------
     In [1]: trimesh.util.round_sigfig(-232453.00014045456, 1)
     Out[1]: -200000.0
@@ -1750,6 +1821,7 @@ def vstack_empty(tup):
         return np.array([])
     return np.vstack(stackable)
 
+
 def write_encoded(file_obj, stuff, encoding='utf-8'):
     '''
     If a file is open in binary mode and a string is passed, encode and write
@@ -1762,12 +1834,12 @@ def write_encoded(file_obj, stuff, encoding='utf-8'):
     encoding: str,          encoding of text
 
     '''
-    binary_file  = 'b' in file_obj.mode
+    binary_file = 'b' in file_obj.mode
     string_stuff = isinstance(stuff, basestring)
     binary_stuff = isinstance(stuff, bytes)
 
     if not _PY3:
-        file_obj.write(stuff)  
+        file_obj.write(stuff)
     elif binary_file and string_stuff:
         file_obj.write(stuff.encode(encoding))
     elif not binary_file and binary_stuff:
@@ -1775,4 +1847,3 @@ def write_encoded(file_obj, stuff, encoding='utf-8'):
     else:
         file_obj.write(stuff)
     file_obj.flush()
-        
